import os
import re
import subprocess
import csv
import datetime
import shutil
import koji

import yaml

from jobtriggers import config
from jobtriggers import exceptions as exc


def log_job(logname, item, item_type, task, arch):
    now = datetime.datetime.now()

    with open(logname, 'a') as csvfile:
        writer = csv.writer(csvfile, delimiter=';')
        writer.writerow([now, item, item_type, task, arch])


def parse_yaml_from_file(filename):
    with open(filename, 'r') as datafile:
        try:
            return yaml.safe_load(datafile.read())
        except yaml.YAMLError, e:
            raise exc.TriggerError(e)


def _should_schedule(formula_file, item_type):
    formula = parse_yaml_from_file(formula_file)
    try:
        run = formula['scheduling']['run']
    except KeyError:
        run = True

    return run and item_type in formula['input']['args']


def get_distgit_branch(release):
    matches = re.findall(r'\.(fc\d{1,2})\.?', release)
    if not matches:
        raise exc.TriggerMsgError('Could not parse disttag from %s' % release)

    return matches[-1].replace('c', '')


def _get_koji_session():
    # for testability reasons...
    return koji.ClientSession("http://koji.fedoraproject.org/kojihub")


def get_distgit_branch_docker(build_id):
    session = _get_koji_session()
    tags = session.listTags(build_id)

    for tag in tags:
        try:
            matches = re.findall(r'^(f\d{1,2})-?.*$', tag['name'])
            if matches:
                return matches[-1]
        except KeyError:
            pass

    raise exc.TriggerMsgError('Could not get disttag for %d' % build_id)


def get_docker_registry_url(build_id):
    session = _get_koji_session()
    archives = session.listArchives(build_id, type='image')

    for archive in archives:
        try:
            urls = archive['extra']['docker']['repositories']
            for url in urls:
                if '@sha' not in url:
                    return url
        except KeyError:
            pass

    raise exc.TriggerMsgError('Could not get docker registry url from %d' % build_id)


def discover_tasks(item_type, repo, branch='master'):
    dirname = repo.split('/')[-1]
    if dirname.endswith('.git'):
        dirname = dirname[:-len(".git")]

    tasks_dir = os.path.join(config.git_cache_dir, dirname)

    # for now, just delete the repo and make a new shallow copy
    if os.path.exists(tasks_dir):
        shutil.rmtree(tasks_dir)

    cmd = ['git', 'clone', '--branch', branch, '--depth', '1', repo, tasks_dir]

    try:
        subprocess.check_output(cmd, stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        # FIXME: this does not really work, since the change in branch is not propagated
        # back, but leaving here for completeness for now
        if 'Could not find remote branch' in e.output and branch != 'master':
            return discover_tasks(item_type, repo, branch='master')
        raise exc.TriggerError(e.output)

    tasks = []
    for task in os.listdir(tasks_dir):
        if task.startswith('.'):
            continue
        taskdir = os.path.join(tasks_dir, task)
        formula_file = os.path.join(taskdir, 'runtask.yml')
        if os.path.isdir(taskdir) and _should_schedule(formula_file, item_type):
            tasks.append(task)

    return tasks
